# Copyright (C) 2023 - 2026 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing the sync tool implementation."""

from fnmatch import filter
from pathlib import Path
import re
import shutil
import tempfile
from typing import List, Union

from git import Repo
from github import Auth, Github, GithubException

from .constants import DEFAULT_BRANCH_NAME, DEFAULT_PULL_REQUEST_TITLE


def include_patterns(*patterns):
    """Include listed patterns in ``copytree()``.

    Factory function that can be used with ``copytree()`` ignore parameter.

    Arguments define a sequence of glob-style patterns
    that are used to specify which files to NOT ignore.

    Creates and returns a function that determines this for each directory
    in the file hierarchy rooted at the source directory when used with
    shutil.copytree().
    """

    def _ignore_patterns(path, names):
        keep = set(name for pattern in patterns for name in filter(names, pattern))
        # Ignore names that are not present in the "keep" set and are not directories
        ignore = set(
            name for name in names if name not in keep and not (Path(path) / name).is_dir()
        )
        return ignore

    return _ignore_patterns


def adapt_regex_from_manifest(accepted_extensions: List[str]) -> List[str]:
    """Adapt regex expressions from manifest read.

    Parameters
    ----------
    accepted_extensions : List[str]
        List of accepted extensions coming from manifest file.

    Returns
    -------
    List[str]
        List of accepted extensions coming from manifest file (adapted).

    """
    acc_regex = []
    for entry in accepted_extensions:
        # Adapt to a proper regex... Especially based on format
        if entry.startswith("*."):
            entry = entry.replace("*.", ".*.", 1)
        acc_regex.append(entry)

    return acc_regex


def delete_folder_contents(
    folder_path: str | Path, accepted_extensions: List[str], clean_to_dir_based_on_manifest: bool
):
    """Delete the content inside a folder without deleting the folder itself.

    Parameters
    ----------
    folder_path : str | Path
        Path to the folder whose content is requested to be deleted.
    accepted_extensions : List[str]
        List of accepted extensions coming from manifest file.
    clean_to_dir_based_on_manifest : bool
        Whether to perform the cleanup of files that match the regex
        in the manifest.

    """
    # Check if the folder exists
    folder = Path(folder_path) if isinstance(folder_path, str) else folder_path
    if not folder.exists():
        print(
            f"Directory '{folder}' does not exist and will not be cleaned - process will continue."
        )
        return

    try:
        # List all files and directories in the folder
        for item_path in folder.iterdir():
            if item_path.is_file():
                if not clean_to_dir_based_on_manifest or any(
                    re.match(entry, item_path.name) is not None for entry in accepted_extensions
                ):
                    # Clean all files if clean_to_dir_based_on_manifest is False,
                    #  otherwise only the ones matching the manifest regex
                    item_path.unlink()

            elif item_path.is_dir():
                # If it's a directory, use recursive call to delete its contents
                delete_folder_contents(
                    item_path, accepted_extensions, clean_to_dir_based_on_manifest
                )
                # After the contents are deleted, remove the directory if it is empty
                # or if manifest-based filtering is off
                if not clean_to_dir_based_on_manifest or not any(item_path.iterdir()):
                    item_path.rmdir()

    except (FileNotFoundError, PermissionError, OSError) as e:  # pragma: no cover
        print(f"An error occurred: {str(e)} - process will continue.")


def synchronize(
    owner: str,
    repository: str,
    token: str,
    from_dir: str | Path,
    to_dir: str | Path,
    include_manifest: str | Path,
    branch_checked_out: str = "main",
    clean_to_dir: bool = False,
    clean_to_dir_based_on_manifest: bool = False,
    dry_run: bool = False,
    skip_ci: bool = False,
    random_branch_name: bool = False,
    target_branch_name: str = DEFAULT_BRANCH_NAME,
    pull_request_title: str = DEFAULT_PULL_REQUEST_TITLE,
) -> Union[str, None]:
    """Synchronize a folder to a remote repository.

    Parameters
    ----------
    owner : str
        Repository owner (user or organization).
    repository : str
        Repository name.
    token : str
        GitHub access token.
    from_dir : str | Path
        Directory from which files want to be synced.
    to_dir : str | Path
        Directory to which files want to be synced (w.r.t. the root of the repository).
    include_manifest : str | Path
        Path to manifest which mentions accepted extension files.
    branch_checked_out : str, optional
        Branch to check out, by default "main".
    clean_to_dir : bool, optional
        Delete the content inside the directory where the files will be synced, by default
        ``False``.
    clean_to_dir_based_on_manifest : bool, optional
        In case ``clean_to_dir`` is requested, perform the cleanup of files that match the regex
        in the manifest. By default, ``False``. If ``clean_to_dir`` is ``False``, this option will
        not have an effect.
    dry_run : bool, optional
        Simulate the behavior of the synchronization without performing it, by default ``False``.
    skip_ci : bool, optional
        Whether to add a ``[skip ci]`` prefix to the commit message or not. By default ``False``.
    random_branch_name : bool, optional
        For testing purposes - generates a random suffix for the branch name.
    target_branch_name : str, optional
        Name of the branch to create for the synchronization, by default it is 'sync/file-sync'.
    pull_request_title : str, optional
        Title of the pull request created after synchronization, by default it is
        'sync: file sync performed by ansys-tools-repo-sync'.

    Returns
    -------
    Union[str, None]
        Pull request URL. In case of dry-run or no files modified, ``None`` is returned.

    """
    # If requested, add random suffix
    if random_branch_name:
        from secrets import token_urlsafe

        target_branch_name = f"{target_branch_name}-{token_urlsafe(16)}"

    # Authenticate with GitHub
    g = Github(auth=Auth.Token(token))

    # Get the repository
    print(f">>> Accessing repository '{owner}/{repository}'...")
    pygithub_repo = g.get_repo(f"{owner}/{repository}")

    # Create a temporary directory for the clone
    #
    # tempfile.TemporaryDirectory will clean itself up once it has run out
    # of scope. No need to actively remove.
    temp_dir = tempfile.TemporaryDirectory(prefix="repo_clone_", ignore_cleanup_errors=True)

    # Retrieve accepted extensions from manifest
    accepted_extensions = []
    include_manifest_path = Path(include_manifest)
    print(f">>> Considering manifest file at {include_manifest_path} ...")
    accepted_extensions = include_manifest_path.read_text().splitlines()

    # Clone the repository
    print(f">>> Cloning repository '{owner}/{repository}'...")
    repo_path = Path(temp_dir.name) / repository
    authenticated_url = f"https://{token}@{pygithub_repo.html_url.split('https://')[-1]}"
    Repo.clone_from(authenticated_url, repo_path)

    # Define the destination path for the files to be synced
    destination_path = repo_path / to_dir
    destination_path.mkdir(parents=True, exist_ok=True)

    # If requested, clean the destination path
    if clean_to_dir:
        print(f">>> Cleaning content inside '{to_dir}'...")
        acc_regex = adapt_regex_from_manifest(accepted_extensions)
        delete_folder_contents(destination_path, acc_regex, clean_to_dir_based_on_manifest)

    # Copy local folder contents to the cloned repository
    print(f">>> Moving desired files from {from_dir} to {destination_path} ...")
    shutil.copytree(
        from_dir,
        destination_path,
        ignore=include_patterns(*accepted_extensions),
        dirs_exist_ok=True,
    )

    print(f">>> Checking out new branch '{target_branch_name}' from '{branch_checked_out}'...")
    repo = Repo(repo_path)
    try:
        # Commit changes to a new branch
        repo.git.checkout(branch_checked_out)
        repo.git.checkout("-b", target_branch_name)
        print(f">>> Committing changes to branch '{target_branch_name}'...")
        repo.git.add("--all")
        repo.index.commit(f"{'[skip ci] ' if skip_ci else ''}sync: add changes from local folder")

        # Get a list of the files modified
        output = repo.git.diff(
            "--compact-summary", f"{branch_checked_out}", f"{target_branch_name}"
        )

        # If output is empty, avoid creating PR
        if not output:
            print(">>> No files to sync... Ignoring PR request.")
            return None
        else:
            print(">>> Summary of modified files...")
            print(output)

        pull_request = None
        if not dry_run:
            # Push changes to remote repositories
            print(f">>> Force-pushing branch '{target_branch_name}' remotely...")
            repo.git.push("--force", "origin", target_branch_name)

            # Create a pull request
            try:
                print(f">>> Creating pull request from '{target_branch_name}'...")
                pull_request = pygithub_repo.create_pull(
                    title=pull_request_title,
                    body="Please review and merge these changes.",
                    base=branch_checked_out,
                    head=target_branch_name,
                )
            except GithubException as err:
                if err.args[0] == 422 or err.data["message"] == "Validation Failed":
                    print(">>> Branch and pull request already existed, searching for it...")

                    # Pull request already exists
                    prs = pygithub_repo.get_pulls()

                    # Find the associated PR (must be opened...)
                    associated_pull_request = None
                    for pr in prs:
                        if pr.head.ref == target_branch_name:
                            associated_pull_request = pr
                            break

                    # Return the associated PR
                    if associated_pull_request:
                        pull_request = associated_pull_request
                    else:
                        # Don't know what could have happened...
                        raise err
                else:
                    # Unidentified error.. raise it.
                    raise err

            print(f">>> Pull request created: {pull_request.html_url}")
            return pull_request.html_url
        else:
            print(">>> Dry run successful.")
            return None
    finally:
        # Close local repo for proper file deletion
        repo.close()



# Copyright (C) 2023 - 2026 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Integration tests for repo synchronization behavior."""

import dataclasses
import shutil
import subprocess

import pytest

from ansys.tools.repo_sync.repo_sync import synchronize

from .conftest import (
    ASSETS_DIRECTORY,
    DEFAULT_SYNC_CONFIG,
    ROOT_PATH,
    SKIP_LOCALLY,
    TOKEN,
    check_files_in_pr,
    cleanup_remote_repo,
    get_pr_from_cli,
)


def test_synchronize():
    """Test synchronization tool (without manifest)."""
    cfg = DEFAULT_SYNC_CONFIG

    # Call the function
    result = None
    try:
        result = synchronize(
            owner=cfg.owner,
            repository=cfg.repository,
            token=TOKEN,
            from_dir=cfg.from_dir,
            to_dir=cfg.to_dir,
            include_manifest=cfg.manifest,
            skip_ci=True,
            random_branch_name=True,
        )

        # Assertions or validations
        assert result is not None
        assert "https://github.com/ansys/ansys-tools-repo-sync/pull/" in result

        # Check that the proper modified files have been added
        list_of_files = ["src/ansys/api/test/v0/hello_world.py", "src/ansys/api/test/v0/test.proto"]
        assert check_files_in_pr(cfg.owner, cfg.repository, result, list_of_files)

    except Exception as err:
        raise err
    finally:
        if result:
            cleanup_remote_repo(cfg.owner, cfg.repository, result)


def test_synchronize_to_existing_pr():
    """Test synchronization tool (when PR already exists)."""
    cfg = DEFAULT_SYNC_CONFIG

    # Call the function
    result = None
    result_pr_already_exists = None
    try:
        result = synchronize(
            owner=cfg.owner,
            repository=cfg.repository,
            token=TOKEN,
            from_dir=cfg.from_dir,
            to_dir=cfg.to_dir,
            include_manifest=cfg.manifest,
            skip_ci=True,
        )

        # Assertions or validations
        assert result is not None
        assert "https://github.com/ansys/ansys-tools-repo-sync/pull/" in result

        # Call the function again - and check that the PR already exists.
        result_pr_already_exists = synchronize(
            owner=cfg.owner,
            repository=cfg.repository,
            token=TOKEN,
            from_dir=cfg.from_dir,
            to_dir=cfg.to_dir,
            include_manifest=cfg.manifest,
            skip_ci=True,
        )

        # Verify the PR is the same
        assert result_pr_already_exists == result

        # Check that the proper modified files have been added
        list_of_files = ["src/ansys/api/test/v0/hello_world.py", "src/ansys/api/test/v0/test.proto"]
        assert check_files_in_pr(cfg.owner, cfg.repository, result, list_of_files)

    except Exception as err:
        raise err
    finally:
        if result:
            cleanup_remote_repo(cfg.owner, cfg.repository, result)
        if result_pr_already_exists and result != result_pr_already_exists:
            cleanup_remote_repo(cfg.owner, result_pr_already_exists, result)


def test_synchronize_with_only_proto_manifest():
    """Test synchronization tool (with manifest)."""
    cfg = dataclasses.replace(
        DEFAULT_SYNC_CONFIG, manifest=ASSETS_DIRECTORY / "manifest_only_proto.txt"
    )

    # Call the function
    result = None
    try:
        result = synchronize(
            owner=cfg.owner,
            repository=cfg.repository,
            token=TOKEN,
            from_dir=cfg.from_dir,
            to_dir=cfg.to_dir,
            include_manifest=cfg.manifest,
            skip_ci=True,
            random_branch_name=True,
        )

        # Assertions or validations
        assert result is not None
        assert "https://github.com/ansys/ansys-tools-repo-sync/pull/" in result

        # Check that the proper modified files have been added
        list_of_files = ["src/ansys/api/test/v0/test.proto"]
        assert check_files_in_pr(cfg.owner, cfg.repository, result, list_of_files)

    except Exception as err:
        raise err
    finally:
        if result:
            cleanup_remote_repo(cfg.owner, cfg.repository, result)


def test_synchronize_no_sync_needed():
    """Test synchronization tool (with manifest referring to non-existing files)."""
    cfg = dataclasses.replace(
        DEFAULT_SYNC_CONFIG, manifest=ASSETS_DIRECTORY / "manifest_no_files.txt"
    )

    # Call the function
    result = None
    try:
        result = synchronize(
            owner=cfg.owner,
            repository=cfg.repository,
            token=TOKEN,
            from_dir=cfg.from_dir,
            to_dir=cfg.to_dir,
            include_manifest=cfg.manifest,
            skip_ci=True,
            random_branch_name=True,
        )

        # Assertions or validations
        assert result is None

    except Exception as err:
        raise err
    finally:
        if result:
            cleanup_remote_repo(cfg.owner, cfg.repository, result)


def test_synchronize_with_cleanup_and_dry_run(capsys):
    """
    Test synchronization tool (with --clean-to-dir flag).

    Notes
    -----
    Executed in dry-run mode.

    """
    cfg = DEFAULT_SYNC_CONFIG

    # Call the function
    result = synchronize(
        owner=cfg.owner,
        repository=cfg.repository,
        token=TOKEN,
        from_dir=cfg.from_dir,
        to_dir=cfg.to_dir,
        include_manifest=cfg.manifest,
        clean_to_dir=True,
        skip_ci=True,
        random_branch_name=True,
        dry_run=True,
    )

    # Assertions or validations
    assert result is None

    # Check stdout
    captured = capsys.readouterr()[0]

    # Search for the modified files
    assert "src/ansys/api/test/v0/hello_world.py" in captured
    assert "src/ansys/api/test/v0/test.proto" in captured
    assert "src/ansys/tools/repo_sync/__init__.py" in captured
    assert "src/ansys/tools/repo_sync/__main__.py" in captured
    assert "src/ansys/tools/repo_sync/repo_sync.py" in captured


def test_synchronize_with_cleanup_based_on_manifest_and_dry_run(capsys):
    """Test synchronization tool with manifest-based cleanup.

    Notes
    -----
    Executed in dry-run mode.

    """
    cfg = dataclasses.replace(
        DEFAULT_SYNC_CONFIG, manifest=ASSETS_DIRECTORY / "manifest_proto_and_init.txt"
    )

    # Call the function
    result = synchronize(
        owner=cfg.owner,
        repository=cfg.repository,
        token=TOKEN,
        from_dir=cfg.from_dir,
        to_dir=cfg.to_dir,
        include_manifest=cfg.manifest,
        clean_to_dir=True,
        clean_to_dir_based_on_manifest=True,
        skip_ci=True,
        random_branch_name=True,
        dry_run=True,
    )

    # Assertions or validations
    assert result is None

    # Check stdout
    captured = capsys.readouterr()[0]

    # Search for the modified files
    assert "src/ansys/api/test/v0/hello_world.py" not in captured
    assert "src/ansys/api/test/v0/test.proto" in captured
    assert "src/ansys/tools/repo_sync/__init__.py" in captured
    assert "src/ansys/tools/repo_sync/__main__.py" not in captured
    assert "src/ansys/tools/repo_sync/repo_sync.py" not in captured


@pytest.mark.skipif(SKIP_LOCALLY, reason="Only runs on workflow")
def test_synchronize_from_cli(tmpdir):
    """Test synchronization tool (without manifest) from CLI."""
    cfg = DEFAULT_SYNC_CONFIG

    # Define a temp directory and copy assets in it
    shutil.copytree(
        ASSETS_DIRECTORY,
        tmpdir,
        dirs_exist_ok=True,
    )

    # Requires installing project
    subprocess.run(
        [
            "pip",
            "install",
            ".",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=ROOT_PATH,
    )

    # Call CLI tool
    completed_process = subprocess.run(
        [
            "repo-sync",
            "--token",
            TOKEN,
            "--owner",
            cfg.owner,
            "--repository",
            cfg.repository,
            "--from-dir",
            cfg.from_dir.name,
            "--to-dir",
            cfg.to_dir,
            "--include-manifest",
            cfg.manifest.name,
            "--skip-ci",
            "--random-branch-name",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=tmpdir,
    )  # ty:ignore[no-matching-overload]

    # Check output info
    print(completed_process.returncode)
    print(completed_process.stdout)
    print(completed_process.stderr)

    # Get the PR associated to the CLI
    pr_url = get_pr_from_cli(cfg.owner, cfg.repository, completed_process.stdout)

    # Check that the proper modified files have been added
    list_of_files = ["src/ansys/api/test/v0/hello_world.py", "src/ansys/api/test/v0/test.proto"]
    assert check_files_in_pr(cfg.owner, cfg.repository, pr_url, list_of_files)

    # Clean up remote repo
    cleanup_remote_repo(cfg.owner, cfg.repository, pr_url)


@pytest.mark.skipif(SKIP_LOCALLY, reason="Only runs on workflow")
def test_synchronize_with_only_proto_manifest_from_cli(tmpdir):
    """Test synchronization tool (with manifest) from CLI."""
    cfg = dataclasses.replace(
        DEFAULT_SYNC_CONFIG, manifest=ASSETS_DIRECTORY / "manifest_only_proto.txt"
    )

    # Define a temp directory and copy assets in it
    shutil.copytree(
        ASSETS_DIRECTORY,
        tmpdir,
        dirs_exist_ok=True,
    )

    # Requires installing project
    subprocess.run(
        [
            "pip",
            "install",
            ".",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=ROOT_PATH,
    )

    # Call CLI tool
    completed_process = subprocess.run(
        [
            "repo-sync",
            "--token",
            TOKEN,
            "--owner",
            cfg.owner,
            "--repository",
            cfg.repository,
            "--from-dir",
            cfg.from_dir.name,
            "--to-dir",
            cfg.to_dir,
            "--include-manifest",
            cfg.manifest.name,
            "--skip-ci",
            "--random-branch-name",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=tmpdir,
    )  # ty:ignore[no-matching-overload]

    # Check output info
    print(completed_process.returncode)
    print(completed_process.stdout)
    print(completed_process.stderr)

    # Get the PR associated to the CLI
    pr_url = get_pr_from_cli(cfg.owner, cfg.repository, completed_process.stdout)

    # Check that the proper modified files have been added
    list_of_files = ["src/ansys/api/test/v0/test.proto"]
    assert check_files_in_pr(cfg.owner, cfg.repository, pr_url, list_of_files)

    # Clean up remote repo
    cleanup_remote_repo(cfg.owner, cfg.repository, pr_url)


@pytest.mark.skipif(SKIP_LOCALLY, reason="Only runs on workflow")
def test_synchronize_with_cleanup_cli(tmpdir):
    """
    Test synchronization tool (with --clean-to-dir flag) from CLI.

    Notes
    -----
    Executed in dry-run mode.

    """
    cfg = DEFAULT_SYNC_CONFIG

    # Define a temp directory and copy assets in it
    shutil.copytree(
        ASSETS_DIRECTORY,
        tmpdir,
        dirs_exist_ok=True,
    )

    # Requires installing project
    subprocess.run(
        [
            "pip",
            "install",
            ".",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=ROOT_PATH,
    )

    # Call CLI tool
    completed_process = subprocess.run(
        [
            "repo-sync",
            "--token",
            TOKEN,
            "--owner",
            cfg.owner,
            "--repository",
            cfg.repository,
            "--from-dir",
            cfg.from_dir.name,
            "--to-dir",
            cfg.to_dir,
            "--include-manifest",
            cfg.manifest.name,
            "--skip-ci",
            "--random-branch-name",
            "--clean-to-dir",
            "--dry-run",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=tmpdir,
    )  # ty:ignore[no-matching-overload]

    # Check output info
    print(completed_process.returncode)
    print(completed_process.stdout)
    print(completed_process.stderr)

    # Check stdout
    captured = completed_process.stdout.decode()

    # Search for the modified files
    assert "src/ansys/api/test/v0/hello_world.py" in captured
    assert "src/ansys/api/test/v0/test.proto" in captured
    assert "src/ansys/tools/repo_sync/__init__.py" in captured
    assert "src/ansys/tools/repo_sync/__main__.py" in captured
    assert "src/ansys/tools/repo_sync/repo_sync.py" in captured


@pytest.mark.skipif(SKIP_LOCALLY, reason="Only runs on workflow")
def test_synchronize_with_no_sync_cli(tmpdir):
    """Test synchronization tool (with no files needed to be synced) from CLI."""
    cfg = dataclasses.replace(
        DEFAULT_SYNC_CONFIG, manifest=ASSETS_DIRECTORY / "manifest_no_files.txt"
    )

    # Define a temp directory and copy assets in it
    shutil.copytree(
        ASSETS_DIRECTORY,
        tmpdir,
        dirs_exist_ok=True,
    )

    # Requires installing project
    subprocess.run(
        [
            "pip",
            "install",
            ".",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=ROOT_PATH,
    )

    # Call CLI tool
    completed_process = subprocess.run(
        [
            "repo-sync",
            "--token",
            TOKEN,
            "--owner",
            cfg.owner,
            "--repository",
            cfg.repository,
            "--from-dir",
            cfg.from_dir.name,
            "--to-dir",
            cfg.to_dir,
            "--include-manifest",
            cfg.manifest.name,
            "--skip-ci",
            "--random-branch-name",
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        cwd=tmpdir,
    )  # ty:ignore[no-matching-overload]

    # Check output info
    print(completed_process.returncode)
    print(completed_process.stdout)
    print(completed_process.stderr)

    # Check stdout
    captured = completed_process.stdout.decode()

    # Search for the modified files
    assert ">>> No files to sync... Ignoring PR request." in captured



# Copyright (C) 2023 - 2026 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Pytest helpers and fixtures for repo sync tests."""

from dataclasses import dataclass
import os
from pathlib import Path

from github import Auth, Github

TEST_PATH = Path(__file__).resolve().parent
ROOT_PATH = TEST_PATH.parent
ASSETS_DIRECTORY = TEST_PATH / "assets"
TOKEN = os.environ.get("TOKEN", "NOTOKEN")
SKIP_LOCALLY = False if os.environ.get("ON_WORKFLOW") else True


@dataclass
class SyncTestConfig:
    """Configuration for a repo-sync test scenario."""

    owner: str
    repository: str
    from_dir: Path
    to_dir: str
    manifest: Path


DEFAULT_SYNC_CONFIG = SyncTestConfig(
    owner="ansys",
    repository="ansys-tools-repo-sync",
    from_dir=ASSETS_DIRECTORY / "ansys",
    to_dir="src/ansys",
    manifest=ASSETS_DIRECTORY / "manifest.txt",
)


def cleanup_remote_repo(owner, repository, pull_request_url):
    """Auxiliary function to clean-up remote repository after tests execution."""
    # Authenticate with GitHub
    g = Github(auth=Auth.Token(TOKEN))

    # Extract owner, repository, and pull request number from the URL
    url_parts = pull_request_url.split("/")
    pull_request_number = int(url_parts[-1])

    # Get the repository
    repo = g.get_repo(f"{owner}/{repository}")

    # Get the Pull Request
    pull_request = repo.get_pull(pull_request_number)

    # Delete the Pull Request
    pull_request.edit(state="closed")

    # Delete the remote branch
    branch_name = pull_request.head.ref
    repo.get_git_ref(f"heads/{branch_name}").delete()


def check_files_in_pr(owner, repository, pull_request_url, list_of_files):
    """Auxiliary function to verify modified files in PR."""
    # Authenticate with GitHub
    g = Github(auth=Auth.Token(TOKEN))

    # Extract owner, repository, and pull request number from the URL
    url_parts = pull_request_url.split("/")
    pull_request_number = int(url_parts[-1])

    # Get the repository
    repo = g.get_repo(f"{owner}/{repository}")

    # Get the Pull Request
    pull_request = repo.get_pull(pull_request_number)

    # Get the list of files changed in the Pull Request
    files_changed = [file.filename for file in pull_request.get_files()]

    # Check that the lists are the same
    return all(item in list_of_files for item in files_changed) and all(
        item in files_changed for item in list_of_files
    )


def get_pr_from_cli(owner, repository, stdout):
    """Auxiliary method to get the PR generated when using CLI tool."""
    # Authenticate with GitHub
    g = Github(auth=Auth.Token(TOKEN))

    # Get the repository
    repo = g.get_repo(f"{owner}/{repository}")

    # Pull request already exists
    prs = repo.get_pulls()

    # Find the branch name
    branch_name = "sync/file-sync"
    elems = stdout.decode("utf-8").split(" ")
    for elem in elems:
        if "sync/file-sync-" in elem:
            branch_name = elem.strip("'")
            break

    # Find the associated PR (must be open...)
    associated_pull_request = None
    for pr in prs:
        if pr.head.ref == branch_name:
            associated_pull_request = pr
            break

    if not associated_pull_request:
        raise RuntimeError("Something went wrong in CLI execution...")
    else:
        return associated_pull_request.html_url



# Copyright (C) 2023 - 2026 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Tool to copy the content of one repo toward an other.

Run with:

.. code::

    repo-sync \
--token <token> \
--owner <organization-name> \
--repository <repository-name> \
--from-dir <path-to-dir-containing-files-to-sync> \
--to-dir <target-dir-for-sync> \
--include-manifest <path-to-manifest>

"""

import click

from .constants import DEFAULT_BRANCH_NAME, DEFAULT_PULL_REQUEST_TITLE
from .repo_sync import synchronize as _synchronize


@click.command(short_help="Copy the content of a repository into an other repository.")
@click.option("--owner", "-o", type=str, help="Name of the owner or organization.", required=True)
@click.option("--repository", "-r", type=str, help="Name of the repository.", required=True)
@click.option("--token", "-t", type=str, help="Personal access token.", required=True)
@click.option(
    "--from-dir",
    type=click.Path(file_okay=False, exists=True),
    help="Path to the folder containing the files to copy.",
    required=True,
)
@click.option(
    "--to-dir",
    type=click.Path(file_okay=False),
    help="Path of the folder that will contain the files (w.r.t. the root of the repository).",
    required=True,
)
@click.option(
    "--include-manifest",
    "-m",
    type=click.Path(dir_okay=False, exists=True),
    help="Manifest to mention accepted extension files.",
    required=True,
)
@click.option("--branch_checked_out", "-b", type=str, help="Branch to check out.", default="main")
@click.option(
    "--clean-to-dir",
    is_flag=True,
    default=False,
    help="Clean the folder defined in --to-dir before synchronizing.",
)
@click.option(
    "--clean-to-dir-based-on-manifest",
    is_flag=True,
    default=False,
    help=(
        "Deletion of target directory is performed based on manifest file"
        " (i.e. only those files matching the extensions on it are deleted)."
        " Only has an effect if --clean-to-dir is passed."
    ),
)
@click.option(
    "--dry-run",
    "-d",
    is_flag=True,
    default=False,
    help="Simulate the behavior of the synchronization without performing it.",
)
@click.option(
    "--skip-ci",
    is_flag=True,
    default=False,
    help="Adds a ``[skip ci]`` prefix to the commit message or not.",
)
@click.option(
    "--random-branch-name",
    is_flag=True,
    default=False,
    help=(
        "Generates a random branch name instead of the typical ``sync/file-sync``. "
        "Used for testing purposes mainly."
    ),
)
@click.option(
    "--target-branch-name",
    type=str,
    default=DEFAULT_BRANCH_NAME,
    help=(
        "Name of the branch to create for the synchronization. "
        f"By default it is '{DEFAULT_BRANCH_NAME}'."
    ),
)
@click.option(
    "--pull-request-title",
    type=str,
    default=DEFAULT_PULL_REQUEST_TITLE,
    help=(
        "Title of the pull request created after synchronization. "
        f"By default it is {DEFAULT_PULL_REQUEST_TITLE}."
    ),
)
def synchronize(
    owner,
    repository,
    token,
    from_dir,
    to_dir,
    include_manifest,
    branch_checked_out,
    clean_to_dir,
    clean_to_dir_based_on_manifest,
    dry_run,
    skip_ci,
    random_branch_name,
    target_branch_name,
    pull_request_title,
):
    """CLI command to execute the repository synchronization."""
    _synchronize(
        owner=owner,
        repository=repository,
        token=token,
        from_dir=from_dir,
        to_dir=to_dir,
        clean_to_dir=clean_to_dir,
        clean_to_dir_based_on_manifest=clean_to_dir_based_on_manifest,
        include_manifest=include_manifest,
        branch_checked_out=branch_checked_out,
        dry_run=dry_run,
        skip_ci=skip_ci,
        random_branch_name=random_branch_name,
        target_branch_name=target_branch_name,
        pull_request_title=pull_request_title,
    )


if __name__ == "__main__":
    synchronize()



"""Sphinx documentation configuration file."""

from datetime import datetime
import os
import shutil

from ansys_sphinx_theme import get_version_match

from ansys.tools.repo_sync import __version__

# Project information
project = "ansys-tools-repo-sync"
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = __version__
cname = os.getenv("DOCUMENTATION_CNAME", "ansys.github.io/ansys-tools-repo-sync")

# Select desired theme, and declare the html title
html_theme = "ansys_sphinx_theme"
html_short_title = html_title = "ansys-tools-repo-sync"

# specify the location of your github repo
html_theme_options = {
    "logo": "pyansys",
    "github_url": "https://github.com/ansys/ansys-tools-repo-sync",
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": get_version_match(__version__),
    },
    "check_switcher": False,
    "ansys_sphinx_theme_autoapi": {
        "project": project,
    },
}

# Sphinx extensions
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx_autodoc_typehints",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
    "ansys_sphinx_theme.extension.autoapi",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    # kept here as an example
    # "scipy": ("https://docs.scipy.org/doc/scipy/reference", None),
    # "numpy": ("https://numpy.org/devdocs", None),
    # "matplotlib": ("https://matplotlib.org/stable", None),
    # "pandas": ("https://pandas.pydata.org/pandas-docs/stable", None),
    # "pyvista": ("https://docs.pyvista.org/", None),
    # "grpc": ("https://grpc.github.io/grpc/python/", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    # "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}


# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# Configuration for Sphinx autoapi
autoapi_type = "python"
autoapi_dirs = ["../../src/ansys/"]
autoapi_options = [
    "members",
    "undoc-members",
    "show-inheritance",
    "show-module-summary",
    "imported-members",
]
autoapi_template_dir = "_autoapi_templates"
autoapi_python_use_implicit_namespaces = True
exclude_patterns = ["_autoapi_templates/index.rst"]

# Copying README images
shutil.copytree("../../images", "images", dirs_exist_ok=True)

# Ignore anchors
linkcheck_anchors = False

# Ignore links
linkcheck_ignore = [
    "https://opensource.org/licenses/MIT",
]
